@echo off 
REM ************************************************************************************
REM Translate an equirectangular input image to a given yaw, pitch, and roll.
REM By Eric Gerds
REM
REM When you run this program, the default is for the yaw translation to occur first,
REM then the pitch, and lastly the roll. These translations must be carried out
REM inseparate and distinct operations. Meaning, if you specify a non zero yaw, pitch,
REM and roll, then PanoTools will run 3 times (3 interpolations will occur).
REM  
REM Given a script, PanoTools executes a translation in the following order:
REM First roll, then pitch, and then yaw.
REM So if you specify a yaw, pitch, roll in a SINGLE script,
REM that is the order in which the translations are carried out.
REM


REM ************************************************************************
REM Initialization
SET searchreal=checkrealnum
goto :initialstuff
:returninitialstuff


REM ************************************************************************
REM Search all input arguments for control parameters
REM such as yaw, pitch, roll and interpolation method. Doing this
REM allows the user to specify control parameters ANYWHERE
REM in the input line, even last.

REM Scan input line for yaw, pitch, roll parameters.
REM Yaw, Pitch, Roll are initialized as "" in this routine
call :getYPR %*

REM Scan the input line for the /r or /R switch.
REM We must initialize Rswitch to blank first.
SET Rswitch=
call :findRswitch %*


REM **************************************************************************
REM Introductory message if no errors
REM Should we clear the screen with cls, or not clear it at all?
REM cls

echo Translate an equirectangular input image to a specified Yaw, Pitch,
echo and Roll using PanoTools.
echo.
echo Input image size must be Width x Height = 2N x N [pixels]
echo.
echo Interpolation method : %interp%
echo.
echo.

REM *******************************************************************************
REM Get Yaw, Pitch, Roll values.
REM If these values have already been specified on the command line, then do nothing.
REM If they have not been specified, then get them from the user now.
REM if Yaw is blank (as initialized by :getYPR), then no values have been specified. 

if "%yaw%" == "" (
     call :inputYPR "Yaw Pitch Roll [/R] (use space between numbers): "
)

REM If no numbers were given, then exit.
if "%yaw%" == "" goto :eof

REM If all 3 coordinates are 0, then exit
if "%yaw%%pitch%%roll%" == "000" goto :eof


REM **************************************************************************************
REM Create script files used by PTStitcher
REM Create scripts for Yaw, Pitch, Roll translation
REM We create a separate script for each to simplify things. If we wanted to do
REM all 3 translations in a single script, we would have to use some complicated
REM equations to determine yaw,pitch, roll. Dos cannot handle such calculations.

SET Scriptyaw=Tscriptyaw.txt
SET Scriptpitch=Tscriptpitch.txt
SET Scriptroll=Tscriptroll.txt

REM If /r switch then put all 3 translations into one script
if "%Rswitch%" == "" goto :transa
   call :make1script %Scriptyaw% %yaw% %pitch% %roll%
   goto :translateloop

:transa
REM If /r switch NOT present, then make translations in 3 separate scripts
REM Create the 3 scripts. Remember to change the sign of yaw, pitch, roll.
REM call :changesign %yaw%
REM call :make1script %Scriptyaw% %validnum% 0 0
call :make1script %Scriptyaw% %yaw% 0 0

REM call :changesign %pitch%
REM call :make1script %Scriptpitch% 0 %validnum% 0
call :make1script %Scriptpitch% 0 %pitch% 0

REM call :changesign %roll%
REM call :make1script %Scriptroll% 0 0 %validnum%
call :make1script %Scriptroll% 0 0 %roll%


REM ******************************************************************************************
:translateloop
REM This is the beginning of a loop. We process each of the input arguments,
REM starting with %1, then %2, etc...

REM We must give all inputs to :searchfirstimage explicitly instead of 
REM using %* because %* cannot be shifted, but the input args themselves will.
SET searchreal=checkrealnum
call :searchfirstimage %1 %2 %3 %4 %5 %6 %7 %8 %9
:shifterloop2
if not "%shiftargs%" == "0" (
   shift /1
   SET /a shiftargs=shiftargs-1
   goto :shifterloop2
)


REM If %1 argument is blank, then stop because we ran out of inputs to process
if "%~1" == "" goto :endtranslate

REM Valid Image now found


REM ***********************************************************************************
REM Naming convention for output files

REM We place output Pano in same directory as the input file %1
SET Panoyaw="%~dpn1(yaw %yaw%).tif"
SET Panopitch="%~dpn1(pitch %pitch%).tif"
SET Panoroll="%~dpn1(roll %roll%).tif"

REM ************************************************************************************
REM Now run PTStitcher to translate the image %1

echo.
echo.

REM Input image to use in next translation
SET tmp="%~1"

REM If /r switch present, then execute the single script
if "%Rswitch%" == "" goto :transb
     SET Panoyaw="%~dpn1(r%roll% p%pitch% y%yaw%).tif"
     if exist %Panoyaw% del %Panoyaw%
     echo [Translating roll %roll%, then pitch %pitch%, then yaw %yaw%  ...]
     %PTStitch% -o %Panoyaw% %Scriptyaw% %tmp%
     goto :conttranslateloop

REM If /r switch not present, then execute the 3 scripts in order
:transb
if "%yaw%" == "0" goto :skiptranslate1

REM Delete if file exists, or else PanoTools PTStitcher gets confused
if exist %Panoyaw% del %Panoyaw%

echo [Translating yaw by %yaw% degrees...]
%PTStitch% -o %Panoyaw% %Scriptyaw% %tmp%
SET tmp=%Panoyaw%



:skiptranslate1
if "%pitch%" == "0" goto :skiptranslate2

REM Delete if file exists, or else PanoTools PTStitcher gets confused
if exist %Panopitch% del %Panopitch%

echo [Translating pitch by %pitch% degrees...]
%PTStitch% -o %Panopitch% %Scriptpitch% %tmp%
SET tmp=%Panopitch%


:skiptranslate2
if "%roll%" == "0" goto :skiptranslate3

REM Delete if file exists, or else PanoTools PTStitcher gets confused
if exist %Panoroll% del %Panoroll%

echo [Translating roll by %roll% degrees...]
%PTStitch% -o %Panoroll% %Scriptroll% %tmp%
SET tmp=%Panoroll%


:skiptranslate3
REM Try to rename file, and delete the intermediate pano files
rename %tmp% "%~n1(y%yaw% p%pitch% r%roll%).tif"

if exist %Panoyaw% del %Panoyaw%
if exist %Panopitch% del %Panopitch%
if exist %Panoroll% del %Panoroll%

:conttranslateloop
REM Continue the loop, and search for additional images to process
shift /1
goto :translateloop


REM **********************************************************************************************
REM Close down this program

:endtranslate
REM No more inputs to process, so we delete the scripts and end program
if exist %Scriptyaw% del %Scriptyaw%
if exist %Scriptpitch% del %Scriptpitch%
if exist %Scriptroll% del %Scriptroll%

REM Exit script
goto :eof


REM *****************************************************************************************************
REM Subroutine to change the sign of a number from positive to negative and vice versa.
REM output is %validnum%
:changesign
SET validnum=%1

REM If only zeros are in the number, then do not change the sign at all.
echo %~1 | findstr /R "[1-9]" > nul
if %errorlevel% equ 0 goto :changesign0
goto :eof

:changesign0
REM Convert positive to negative number
echo %~1 | findstr /R "^+" > nul
if %errorlevel% gtr 0 goto :changesign1
    SET validnum=%validnum:+=-%
    goto :eof

:changesign1
REM Convert negative to positive
echo %~1 | findstr /R "^-" > nul
if %errorlevel% gtr 0 goto :changesign2
    SET validnum=%validnum:-=+%
    goto :eof

:changesign2
REM If no sign present, then add a minus sign
    SET validnum=-%validnum%

REM Return from subroutine
goto :eof


REM **************************************************************************************************************
REM Subroutine - Create a script. %1 is the script name.
REM %2, %3, %4 are yaw, pitch, roll specifications
:make1script

REM If we specify no output width, then output is same size as input

echo # input equirectangular image > "%~1"
echo o f4 y%~2 p%~3 r%~4 v360 >> "%~1"
echo. >> "%~1"  
echo # output equirectangular image >> "%~1"
echo p f2 v360 n"TIFF" >> "%~1"
echo. >> "%~1" 
echo # The 'm' line sets the interpolation parameters >> "%~1"
echo # i0=poly3, i1=spline16, i2=spline36, i4=spline64, i3=sinc256, i7=sinc1024 >> "%~1"
echo m g1 %interpolate% >> "%~1"
echo. >> "%~1"
 
goto :eof


REM ***************************************************************************************************************
REM SUBROUTINE  - Get keyboard input for %roll%, %pitch%, %yaw%. Verify all input characters.
REM These 3 numbers are real numbers, and can be positive, negative, zero, non-integer, integer.
REM
REM This program is actually a subroutine that is called from another DOS batch routine.
REM INPUT is %1 argument, which is the input message displayed.
REM Output is %roll%, %pitch%, %yaw%

REM Input Yaw, Pitch, Roll
:inputYPR 

REM For some reason, I need to clear the variable using 'set num3='
REM first before getting input with 'set /p num3 =', because if an invalid input is given the first time, 
REM that value remains in num3 if we execute 'set /p num3 =' a second time and just press the
REM carriage return as input.
set num3=
set /p num3=%~1


REM If num3 is completely empty, then exit this routine.
REM This is not so easy to check for, however, because num3 could have multiple words
REM separated by spaces, and may contain one or more sets of quotes.
REM 
REM If we just use    'if %num3% =='  we could get an error when num3 contains more
REM than one word. Normally, using quotes would avoid that.
REM But, if we use  'if "%num3%" =='  we could get an error if num3 contains quotes itself.
REM The most reliable method is to look at only the first word in num3 by calling :getfirstarg
REM and then testing that first word to see if it is empty or not.
call :getfirstarg %num3%
if a%firstarg%a == aa goto :eof


REM Note here that we do NOT use quotes around num3 when calling :getYPR.
REM The effect is that if num3 contains words separated by spaces, then each word is 
REM sent as a separate input argument to :getYPR. 
REM The only problem here comes up if num3 contains quotes, which I would like
REM to remove when sending it to :getYPR. We remove any quotes by using %num3:"=%
REM So then if there are any leading/trailing spaces, they are removed when calling :getYPR,
REM The only hole here is if num3 contains one single " character, which will cause this
REM program to exit.
call :getYPR %num3:"=% 

REM yaw is assumed to be the first input real number
REM If no valid number, give error message
if "%yaw%" == ""  (
   echo Invalid input numbers.
   echo Please input Yaw, Pitch, Roll - [in that order].
   echo Numbers can be positive, negative, integer, non-integer.
   echo If you input a negative number, make sure there are no spaces
   echo       between the '-' sign and the number.
   echo.
   echo Note: Yaw is a side to side movement, Pitch is an up/down movement. 
   echo Yaw can be specified between [-180, 180] or [0, 360] or [-360, 0] degrees
   echo Pitch can be specified between [-90, 90] degrees
   echo Roll can be specifed between [-180, 180] or [0, 360] or [-360, 0] degrees
   echo Example:   25.4  -170  0     
   echo.   
   goto :inputYPR
   
)


REM Scan the input line for the /r or /R switch.
call :findRswitch %num3:"=%

REM Return from subroutine
goto :eof

REM ***********************************************************************************************************
REM Search for /r or /R switch in the input args
REM 'if /i' means to ignore case.
:findRswitch


:findRswitch1
if "%~1" == "" goto :eof
if /i "%~1" == "/R" (
    SET Rswitch=on
    goto :eof
)

shift /1
goto :findRswitch1

REM ***********************************************************************************************************
REM Search all input parameters to this subroutine, and return the first 3 real numbers found.
REM Return the results in %yaw%, %pitch%, %roll% - IN THAT ORDER.
REM If no valid numbers found, then all 3 output variables are blank.
REM If yaw is given a value but not pitch,roll then pitch,roll = 0.
REM If yaw,pitch are gievn values but not roll then roll = 0.
:getYPR

SET yaw=
SET pitch=
SET roll=

:getYPR0

REM Get yaw
if "%~1" == "" goto :eof
call :checkrealnum %~1
shift /1
if "%validnum%" == "" goto :getYPR0
SET yaw=%validnum%

REM If yaw is given a valid number, then set pitch,roll = 0 in case
REM they are not given any values at all.
SET pitch=0
SET roll=0

:getYPR1

REM Get pitch
if "%~1" == "" goto :eof
call :checkrealnum %~1
shift /1
if "%validnum%" == "" goto :getYPR1
SET pitch=%validnum%


:getYPR2

REM Get roll
if "%~1" == "" goto :eof
call :checkrealnum %~1
shift /1
if "%validnum%" == "" goto :getYPR2
SET roll=%validnum%


REM Return from subroutine
goto :eof

REM *******************************************************************************************************
:instruct
REM Print out version number
goto dfpversion
:returndfpversion

echo This program will translate one or more equirectangular images 
echo [of size Width x Height = 2N x N] to a new Yaw, Pitch, 
echo and Roll coordinate using PanoTools. Yaw, Pitch, and Roll
echo are in units of degrees.

echo.
echo Yaw is translated first, THEN Pitch, and THEN roll.
echo If you supply a '/R' along with Yaw, Pitch, Roll
echo then Roll is translated first, then Pitch, and then Yaw.
echo Example: If you translated an image using yaw pitch roll = 20 30 40
echo you could then undo that translation using -20 -30 -40 /R
echo.
echo The output image(s) will have the same width and height as
echo the input image(s). 

echo.
echo No antialiasing filter is used in creating the output images.
echo.
pause
echo.
echo To run this program from the command line type:
echo "[path]%~nx0" [Interpolator] [Yaw] [Pitch] [Roll] [/R]
echo     [input image 1][input image 2]...[input image N]
echo.
echo A maximum of 9 input arguments can be given to this program.
echo The interpolator choices are as follows:
echo       poly3   spline16   spline36  spline64  sinc256  or  sinc1024
echo Default interpolation is %interp%.
echo.
echo /R will cause the order of the translation to be reversed:
echo first Roll, then Pitch, and then Yaw.
echo.
echo To run this program from Windows, just drag an equirectangular image 
echo onto this program's icon or onto a shortcut pointing to this program.
echo (You can select MULTIPLE images simultaneously and drag them
echo onto the program icon. All the selected images will be translated to
echo the new coordinates)
echo.
echo If you wish, you can create a shortcut pointing to this program,
echo and place the shortcut in your Windows 'Send To' folder. 
echo On Windows XP it would be located at   ~%USERNAME%\SendTo\ 
echo Then all you have to do is right-click on any image(s) you wish to resize,
echo and choose 'SendTo' this program.
echo.
pause
goto done


REM *************************************************************************************************************
REM ************************************* Library of Subroutines *********************************************
REM *************************************************************************************************************
REM *************************************************************************************************************
REM *************************************************************************************************************
REM *************************************************************************************************************


REM Warnings about using 'if' statements with input arguments %1, %2, %3, etc...

REM Often times,  'if %var% =='   statement can give a program error because %var%
REM may contain spaces with multiple words. This confuses the if statement, unless
REM you are absolutely sure that %var% will not contains spaces.
REM You can get around this by using   if "%var%" ==
REM But, if var itself contains multiple words with spaces contained by quotes,
REM then     if "%var%" ==  will cause problems for you as well!
REM By convention, we will use  if "%var%"   in this program, only make sure that
REM our internal variables here do not contains multiple words with spaces contained by quotes.
REM And if you know that an internal variable is guaranteed to have quotes, then
REM simply use  if %var% ==    instead of   if "%var%" ==
REM Complicated, but that is DOS for you!

REM Note, however, that %1 will often already have string quotation marks in it.
REM I cannot use   if "%1" ==    statement because %1 can contain quotes itself
REM and this screws up the program sometimes, so we use another way.
REM Instead we can use   if "%~1" == 
REM Note that ~1 will remove any quotes (if any), and then "%~1" will guarantee
REM that only one set of quotes are present.

REM Another danger:
REM If %1 contains parentheses in it ()
REM and you use an 'if' statement that uses parentheses such as:
REM                if %1 == jjjhj (
REM                   command %1
REM                    )
REM 
REM We would get an error here as well! So, we need to be VERY careful with our if statement.
REM In the above case, just use quotes so any parentheses in %1 do not get mixed up with the
REM parentheses of the 'if' statement:
REM               if "%~1" == jjjhj (
REM                   command "%~1"
REM                      )


REM ***********************************************************************************************************
REM Print out the current version number of Dos for Panos
REM Usage:
REM       goto dfpversion
REM       :returndfpversion
REM
:dfpversion
echo DOS Utilities for Panos (DOSUP) v1.1b  By Eric Gerds  http://www.pinlady.net/vr
echo.
goto returndfpversion


REM *****************************************************************************************************
REM This is stuff that needs to be done at the very beginning of each Dos utility.
REM The variable 'searchreal' must be set=checknum or checkrealnum 
REM before running this subroutine.
:initialstuff

REM Default interpolation setting
REM The 'm' line in script sets the interpolation parameters
REM poly3 is worst, sinc1024 is best - except that sinc causes ringing at high contrast edges
REM i0=poly3, i1=spline16, i2=spline36, i4=spline64, i3=sinc256, i7=sinc1024
SET interpolate=i4
SET interp=spline64


REM Verify that command interpreter is cmd.exe and that OS is
REM Win 2000/XP 
goto verifyos
:returnverifyos


REM Verify that there are any input arguments at all to this program
REM If there were no input arguments, then give message and exit.
if "%~1" == "" goto instruct


REM Process command line '/?' argument
REM Display purpose of program and command line argument(s)
REM Check for /? BEFORE cls command because this means you are
REM already in DOS mode when you started this program.
REM We check ALL args for /? because a "call [subroutine] /?" later on would
REM mess up the program. I check each arg one by one just to be safe.
REM Do not use any 'for' loop or other fancy stuff to test for /? because it won't work.
if "%~1" == "/?" goto instruct
if "%~2" == "/?" goto instruct
if "%~3" == "/?" goto instruct
if "%~4" == "/?" goto instruct
if "%~5" == "/?" goto instruct
if "%~6" == "/?" goto instruct
if "%~7" == "/?" goto instruct
if "%~8" == "/?" goto instruct
if "%~9" == "/?" goto instruct


REM Check validity of PTStitcher
REM %PTStitch% is assumed to have quotes
goto getptstitcher
:returngetptstitcher


REM Search all input arguments for interpolation setting
call :getinterp %*


REM Now search for the first VALID image input argument.
REM Note that 'shift /1' will shift the input args %1 %2 %3 ..., but %* itself will NOT be shifted.
REM SET searchreal=checknum or checkrealnum   before calling  :searchfirstimage
call :searchfirstimage %*
:shifterloop1
if "%shiftargs%" neq "0" (
   shift /1
   SET /a shiftargs=shiftargs-1
   goto :shifterloop1
)

REM No input images found, so show instructions and exit program
if "%~1" == "" goto instruct

REM Valid Image now found

REM We set the default fov for all cube faces = 90 degrees
SET cubefov=90


REM Return from subroutine
goto :returninitialstuff


REM *****************************************************************************************************
REM Subroutine - Search the input line for the first input image found.
REM The output is 'shiftargs' which tells you by how many args to shift the input line.
REM Add this to the Library.
REM Input var is %searchreal% - which should = checkrealnum OR checknum
:searchfirstimage

SET shiftargs=0

:searchfirstimag1
REM If %1 argument is blank, then stop because we ran out of inputs to process
if "%~1" == "" goto :eof

if "%~1" == "/?" goto :searchfirstimag2

REM If we detect /r or /R then skip it, because we already scanned for this.
if /i "%~1" == "/R" goto :searchfirstimag2

REM If %1 is a real number, then skip it since we already scanned the input arguments
REM for yaw, pitch, roll.
REM We remove any quotes to checknum input using ~1
call :%searchreal% %~1
if "%validnum%" neq "" goto :searchfirstimag2

REM If %1 is interpolation method specification, then skip it, since
REM we already scanned the input arguments for interpolation method.
REM We remove any quotes to input for checkinterp
call :checkinterp %~1
if "%tmpinterpolate%" neq "" goto :searchfirstimag2

REM Check validity of Input Image %1
REM If Image file was specified but does not exist, then give error message
REM and then continue processing the next input argument.
if not exist "%~1" (
   call :mess3 "%~1"
   goto :searchfirstimag2
)

REM Verify the .tif .jpg .bmp extension for the input file %1
REM > nul will prevent the find command from outputting stuff to screen.
REM /I parameter means to ignore case.
echo "%~x1" | findstr /i ".tif .jpg .bmp" > nul
if %errorlevel% gtr 0 (
      call :notimage "%~1"
      goto :searchfirstimag2
)

REM Valid image found, so return from subroutine
goto :eof

:searchfirstimag2
SET /a shiftargs=shiftargs+1
shift /1
goto :searchfirstimag1


REM ***********************************************************************************************************
REM Check if PTStitcher is being registered. [This is when the user specifies the location of
REM PTStitcher on their hard drive]
REM If so, save it's path for later use with Dos for Panos.
REM The path will be saved in a DOS program pointed to by the variable 'dfpstitcher'
REM 'dfpstitcher' should point to a location that is always accessible, regardless of the current
REM working directory of Dos for Panos, or the value of %PATH%. 
REM %windir% is a good location, for example.
REM
REM If PTStitcher is not being registered, then find the path of PTStitcher.
REM If path not found, then give error message.
REM
REM Run this routine AFTER you are sure that cmd.exe is running and command extensions are on.
REM The return from this routine is :returngetptstitcher
REM 
REM To call this subroutine, type the following 2 lines
REM         goto getptstitcher
REM        :returngetptstitcher
:getptstitcher


REM Location of DOS helper that sets the value of 'PTStitch' variable,
REM which contains the PTStitcher.exe path.
SET dfpstitcher="%windir%\Dos4PanosStPath.bat"

REM PTStitcher will only be registered if it is given as the ONLY input argument

REM If no input args, then PTStitcher is NOT being registered.
REM So, see if PTStitcher exists and we can get it's path.
if "%~1" == "" goto getptstitcher1


SET tmp=

REM Determine if 'PTStitcher.exe' given as any one of the input file arguments.
for %%A in ("%~7" "%~6" "%~5" "%~4" "%~3" "%~2" "%~1") do (
    if /i "%%~nxA" == "PTStitcher.exe" SET tmp="%%~A"
)

REM If not, see if PTStitcher exists and we can get it's path.
REM %tmp% will contain quotes, so we cannot use quotes in this 'if' statement.
if a%tmp%a == aa goto :getptstitcher1


REM If we reach this point, then PTStitcher.exe is given as an input argument
REM with the intent that it be 'registered' with Dos For Panos.

REM Must use quotes such that PTStitch=" "
echo REM Set path of PanoTools PTStitcher.exe so it can work with Dos Utilities for Panos >  %dfpstitcher%
echo REM Dos Utilities For Panos  By Eric Gerds     http://www.pinlady.net/vr >>  %dfpstitcher%
echo. >>  %dfpstitcher%
echo SET PTStitch=%tmp% >>  %dfpstitcher%
echo. >>  %dfpstitcher%

cls
echo The path of PTStitcher is now registered as:
echo %tmp%
echo.
echo If this is the correct path, then Dos Utilities for Panos is ready to be used.
echo.
pause


REM PTStitcher has been registered, so now exit the program
goto :eof


REM Find PTStitcher.exe and get it's path. If not found, give error message.
:getptstitcher1

SET PTStitch=""

REM See if Dos program that sets PTStitcher path exists
if not exist %dfpstitcher% goto nostitcher

REM It exists, so run the file to set the stitcher path
call %dfpstitcher%

if %PTStitch% == "" goto nostitcher

REM PTStitcher found, so return from this subroutine
if exist %PTStitch% goto returngetptstitcher


REM Stitcher.exe not found, so give error message and then exit the program
:nostitcher
cls
echo Error! Cannot find PanoTools PTStitcher.exe
echo.
echo 'Dos Utilities For Panos' requires the correct path
echo to PTStitcher.exe in order to function properly.
echo. 
echo Please drag PTStitcher.exe onto any one of the Dos 
echo programs, and then try again.
echo.
echo.
echo.
pause 

REM Exit the program
goto :eof


REM *********************************************************************************************************
:error1
echo.
echo Error...
echo The resize scriptfile '%Script%' is missing.
echo Resizing of input image is aborted.
echo.
pause
goto :eof


REM ***************************************************************************************
REM SUBROUTINE - output error message
:mess3
echo.
echo Error...
echo Input argument does not appear to be valid.
echo Skipping argument: "%~1"
echo.
pause
echo.

REM Return from subroutine
goto :eof

REM *****************************************************************************************
REM SUBROUTINE
:notimage

REM %~nx1 means that for image %1, 
REM display only file name and extension without the path
echo.
echo Error...
echo File does not appear to have an acceptable  
echo extension such as .tif .jpg or .bmp
echo Skipping file: "%~nx1"
echo.
pause
echo.

REM Return from subroutine
goto :eof

REM ******************************************************************************************************
REM SUBROUTINE - Return only the first input argument, including any possible quotes
:getfirstarg
SET firstarg=
SET firstarg=%1
goto :eof

REM *******************************************************************************************************
REM SUBROUTINE  - Get keyboard input for positive integer. Verify all input characters.
REM This program is actually a subroutine that is called from another DOS batch routine.
REM INPUT is %1 argument, which is the input message displayed.
REM Output is %posnum%

:getnum
REM For some reason, the regular expressions in this DOS version do not work
REM completely correctly. For example, I cannot detect 5*5 as an invalid input
REM even though I would give the proper regular expression to detect this.
REM I believe that DOS is a little buggy in this regard. 

REM For some reason, I need to clear the variable using 'set posnum='
REM first before getting input with 'set /p posnum =', because if an invalid input is given the first time, 
REM that value remains in posnum if we execute 'set /p posnum =' a second time and just press the
REM carriage return as input.
set posnum=
set /p posnum=%~1

REM If posnum is completely empty, then exit this routine.
REM This is not so easy to check for, however, because posnum could have multiple words
REM separated by spaces, and may contain one or more sets of quotes.
REM 
REM If we just use    'if %posnum% =='  we could get an error when posnum contains more
REM than one word. Normally, using quotes would avoid that.
REM But, if we use  'if "%posnum%" =='  we could get an error if posnum contains quotes itself.
REM The most reliable method is to look at only the first word in posnum by calling :getfirstarg
REM and then testing that first word to see if it is empty or not.
call :getfirstarg %posnum%
if a%firstarg%a == aa goto finish

REM Note here that we do NOT use quotes around posnum when calling checknum.
REM The effect is that if posnum contains words separated by spaces, then each word is 
REM sent as a separate input argument to checknum. If checknum detects more than one 
REM input argument, then obviously posnum is not a valid single word number.
REM The only problem here comes up if posnum contains quotes, which I would like
REM to remove when sending it to checknum. We remove any quotes by using %posnum:"=%
REM So then if there are any leading/trailing spaces, they are removes when calling checknum,
REM and if there is more than one word in posnum, that will tell checknum that posnum is
REM not a valid 'single word' number.
REM The only hole here is if posnum contains one single " character, which will cause this
REM program to exit.
call :checknum %posnum:"=% 
if "%validnum%" == "" (
   echo wrong numerical input!
   echo The only permitted characters are 0 thru 9.  
   echo No commas or decimal points or other characters.
   echo.
   goto getnum
)

SET posnum=
SET posnum=%validnum%

REM Return from subroutine
:finish
set ttt=
goto :eof


REM *******************************************************************************************************
REM SUBROUTINE - Check if input parameter %1 is a positive number. Return integer value.
REM
REM Output is %validnum%, which will contain the INTEGER part of the positive number. 
REM So when you are done calling this routine, just use %validnum% as your number. 
REM If validnum is blank, then the input was not a positive number.
REM
REM It is better if the %1 has NO surrounding quotes in it, that way any leading/trailing spaces
REM on the input are automatically removed. 
REM To ensure this, you can call this routine using:
REM call :checknum %var:"=%      or
REM call :checknum %~1              if using an input argument from a routine.             
REM
REM This way, if the input to this routine has more than one word in it, the words are assigned 
REM to %2, %3 etc..., which means that the input variable to this routine had more than
REM one word, and so is not a valid single number.
REM Check if a %2 exists. If so, then there was more than one word as input.
REM
REM BUG NOTE:  findstr /R "[^0-9]" does not work correctly for some reason, but 
REM findstr /R "^[^0-9]" and findstr /R "$[^0-9]"    do work.
 
:checknum

REM echo.
REM echo %1

set validnum=
REM We use quotes around %1, just in case we forget to remove them when calling this routine.
REM This avoids an error in the 'if' statement.
REM Note: We cannot say   'if %~1 ==' 
REM since %1 may be a string with multiple words, which gives an error.
if "%~1" == "" goto :eof

REM If %2 is not blank, then we have a multiword input, so invalid number
if "%~2" neq "" goto :eof

REM Make sure there are numerical digits in %1
REM We use regular expression here.
REM Note, I use %1 and not %~1 because it is ok to keep the
REM surrounding quotes that %1 may have in the next regular expression.
echo %~1 | findstr /i "0 1 2 3 4 5 6 7 8 9" > nul
if %errorlevel% gtr 0 goto :eof

REM We remove all numeric digits from the number
REM If anything is left over, then we have a non number.
REM We have to remove spaces as well.
SET t1=%~1
if "%t1%" neq "" SET t1=%t1:0=%
if "%t1%" neq "" SET t1=%t1:1=%
if "%t1%" neq "" SET t1=%t1:2=%
if "%t1%" neq "" SET t1=%t1:3=%
if "%t1%" neq "" SET t1=%t1:4=%
if "%t1%" neq "" SET t1=%t1:5=%
if "%t1%" neq "" SET t1=%t1:6=%
if "%t1%" neq "" SET t1=%t1:7=%
if "%t1%" neq "" SET t1=%t1:8=%
if "%t1%" neq "" SET t1=%t1:9=%
if "%t1%" neq "" SET t1=%t1:.=%
if "%t1%" neq "" SET t1=%t1:+=%
if "%t1%" neq "" SET t1=%t1: =%

REM If anything left over, even a minus sign, then we do not have a positive number
if "%t1%" neq "" goto :eof

REM Our routine does not properly deal with -0 or -00 etc...
REM But this is ok for now

REM More than one '.' in string is error
echo %~1 | findstr /R "\..*\." > nul
if %errorlevel% equ 0 goto :eof

REM + sign can ONLY be at beginning of string
echo %~1 | findstr /R ".+" > nul
if %errorlevel% equ 0 goto :eof


SET t1=
SET t1=%~1

REM Remove + sign, not needed
if "%t1%" neq "" SET t1=%t1:+=%


REM Remove leading zeros from number
REM Leading zeros interpreted as HEX or OCTAL BASE?
:checknum1
REM /R "^0." does not seem to work correctly, so we use /R "^0[0-9]"
echo %t1% | findstr /R "^0[0-9]" > nul
if %errorlevel% gtr 0 goto :checknum2
SET t1=abc%t1%
SET t1=%t1:abc0=%
goto :checknum1
:checknum2


REM If number=.XXXX we convert to number=0.XXXXX
REM We do this conversion BEFORE taking integer value of number.
echo %t1% | findstr /R "^\." > nul
if %errorlevel% equ 0 SET t1=0%t1%

REM Is there a way to take the integer value of a decimal number? Yes.
REM Replace decimal point with space, then call subroutine that
REM returns the first argument.
REM Replace decimal point (.) with space
if "%t1%" neq "" SET t1=%t1:.= %
call :getfirstarg1 %t1%


REM Return from subroutine
goto :eof

REM Return first argument without any quotes
:getfirstarg1
SET validnum=%~1
goto :eof



REM *******************************************************************************************************
REM SUBROUTINE - Check if input parameter %1 is a number (positive, negative, 
REM integer, non integer, zero). Decimal numbers are assumed to have '.' and NOT ','
REM
REM Output is that real number in the variable %validnum%.  
REM So when you are done calling this routine, just use %validnum% as your number. 
REM If validnum is blank, then the input was not a valid number.
REM
REM It is better if the %1 has NO surrounding quotes in it, that way any leading/trailing spaces
REM on the input are automatically removed. 
REM To ensure this, you can call this routine using:
REM call :checkrealnum %var:"=%      or
REM call :checkrealnum %~1              if using an input argument from a routine.             
REM
REM This way, if the input to this routine has more than one word in it, the words are assigned 
REM to %2, %3 etc..., which means that the input variable to this routine had more than
REM one word, and so is not a valid single number.
REM Check if a %2 exists. If so, then there was more than one word as input.
REM
REM BUG NOTE:  findstr /R "[^0-9]" does not work correctly for some reason, but 
REM findstr /R "^[^0-9]" and findstr /R "$[^0-9]"    do work.
 
:checkrealnum

REM echo.
REM echo %1

set validnum=
REM We use quotes around %1, just in case we forget to remove them when calling this routine.
REM This avoids an error in the 'if' statement.
REM Note: We cannot say   'if %~1 ==' 
REM since %1 may be a string with multiple words, which gives an error.
if "%~1" == "" goto :eof

REM If %2 is not blank, then we have a multiword input, so invalid number
if "%~2" neq "" goto :eof

REM Make sure there are numerical digits in %1
REM We use regular expression here.
REM Note, I use %1 and not %~1 because it is ok to keep the
REM surrounding quotes that %1 may have in the next regular expression.
echo %~1 | findstr /i "0 1 2 3 4 5 6 7 8 9" > nul
if %errorlevel% gtr 0 goto :eof

REM We remove all numeric digits from the number
REM If anything is left over, then we have a non number.
REM We have to remove spaces as well.
SET t1=%~1
if "%t1%" neq "" SET t1=%t1:0=%
if "%t1%" neq "" SET t1=%t1:1=%
if "%t1%" neq "" SET t1=%t1:2=%
if "%t1%" neq "" SET t1=%t1:3=%
if "%t1%" neq "" SET t1=%t1:4=%
if "%t1%" neq "" SET t1=%t1:5=%
if "%t1%" neq "" SET t1=%t1:6=%
if "%t1%" neq "" SET t1=%t1:7=%
if "%t1%" neq "" SET t1=%t1:8=%
if "%t1%" neq "" SET t1=%t1:9=%
if "%t1%" neq "" SET t1=%t1:.=%
if "%t1%" neq "" SET t1=%t1:+=%
if "%t1%" neq "" SET t1=%t1:-=%
if "%t1%" neq "" SET t1=%t1: =%

if "%t1%" neq "" goto :eof


REM More than one '.' in string is error
echo %~1 | findstr /R "\..*\." > nul
if %errorlevel% equ 0 goto :eof

REM + sign can ONLY be at beginning of string
echo %~1 | findstr /R ".+" > nul
if %errorlevel% equ 0 goto :eof

REM - sign can ONLY be at beginning of string
echo %~1 | findstr /R ".\-" > nul
if %errorlevel% equ 0 goto :eof

REM See if any non zero numerical digits present (1-9)
REM If none present, but we know that numerical digits ARE present,
REM then they must be all zeros.
echo %~1 | findstr /R "[1-9]" > nul
if %errorlevel% gtr 0 (
     SET validnum=0
     goto :eof  
)


SET t1=
SET t1=%~1


REM AFTER checking for + at start of string, remove + sign (not needed)
if "%t1%" neq "" SET t1=%t1:+=%

REM Remove leading zeros from number
REM Leading zeros interpreted as HEX or OCTAL BASE?
:checkrealnum1
REM /R "^0." does not seem to work correctly, so we use /R "^0[0-9]"
echo %t1% | findstr /R "^0[0-9]" > nul
if %errorlevel% gtr 0 goto :checkrealnum2
SET t1=abc%t1%
SET t1=%t1:abc0=%
goto :checkrealnum1
:checkrealnum2


REM Remove trailing zeros after decimal point
REM If number=XXXX.Y000 we convert to number=XXXX.Y 
:checkrealnum3
SET t1=%t1%abc
echo %t1% | findstr /R "\.[0-9][0-9]*0abc" > nul
if %errorlevel% gtr 0 goto :checkrealnum4 
    SET t1=%t1:0abc=%
    goto :checkrealnum3
:checkrealnum4

REM At this point, t1 still ends with 'abc', but that is ok because we use this below.

REM If number=XXXX. we convert to number=XXXXX.0
REM Bug in DOS prevents findstr /R "[0-9]*\.$" from working.
REM So we try this trick below.
SET t1=%t1:.abc=.0%
SET t1=%t1:abc=%


SET validnum=%t1%
REM Return from subroutine
goto :eof


REM *******************************************************************************************************
REM SUBROUTINE - Check for width and cube face fov specification in the input arguments.
REM Input is ALL the input arguments to this program %1, %2, ... %N
REM If valid numbers found, then the 'width' variable and 'cubefov' variables will be set.

:getpanowidth
if "%~1" == "" goto :eof

REM Get width, must be > 0, integer
REM We use ~1 to remove any possible quotes.
call :checknum %~1
shift /1
if "%validnum%" == ""  goto :getpanowidth

SET width=
SET width=%validnum%

:getpanowidth1
if "%~1" == "" goto :eof

REM Get cube face fov.
REM fov must be real number, >0 and < 180.
REM But since 'if' does not work with decimal point, we call :checknum instead of 
REM checkrealnum to make sure we have a number, and get back a positive integer
call :checknum %~1
if "%validnum%" == "" (
    shift /1
    goto :getpanowidth1
)

REM Does 'if' work on numbers with a decimal point? No.
REM So we had checknum return the integer part of the number.
if %validnum% gtr 179 goto :eof
if %validnum% lss 0 goto :eof

REM Now we call checkrealnum, which will format the real number for us.
call :checkrealnum %~1
if "%validnum%" == "" goto :eof

SET cubefov=%validnum%

REM Return from subroutine
goto :eof


REM *******************************************************************************************************
REM SUBROUTINE - get interpolation parameter anywhere in the input line.
REM Input is ALL the input arguments to this program %1, %2, ... %N

:getinterp
if "%~1" == "" goto :eof

REM Check %1, if it is interpolator param, then set variables and exit routine
REM We remove any quotes to input for checkinterp
call :checkinterp %~1
if "%tmpinterpolate%" neq "" (
   SET interpolate=%tmpinterpolate%
   SET interp=%tmpinterp%
   goto :eof
)

REM Else shift the paramaters to check the next input argument
shift /1
goto getinterp

REM Return from subroutine
goto :eof

REM *******************************************************************************************************
REM SUBROUTINE - Check for interpolator specification in the input argument %1
REM Input is %1, output is %tmpinterpolate%, %tmpinterp%
REM if the %1 input is poly3, spline16, etc... then set the output variables accordingly.

:checkinterp
set tmpinterpolate=
set tmpinterp=

REM If more than one input argument, then error
if "%~2" neq "" goto :eof

REM Check for interpolation parameter, case insensitive search.
REM i0=poly3, i1=spline16, i2=spline36, i4=spline64, i3=sinc256, i7=sinc1024
if /i "%~1" == "poly3" (
   SET tmpinterpolate=i0
   SET tmpinterp=poly3
   goto :eof
 )
if /i "%~1" == "spline16" (
   SET tmpinterpolate=i1
   SET tmpinterp=spline16
   goto :eof
 )
if /i "%~1" == "spline36" (
   SET tmpinterpolate=i2
   SET tmpinterp=spline36
   goto :eof
 )
if /i "%~1" == "spline64" (
   SET tmpinterpolate=i4
   SET tmpinterp=spline64
   goto :eof
 )
if /i "%~1" == "sinc256" (
   SET tmpinterpolate=i3
   SET tmpinterp=sinc256
   goto :eof
 )
if /i "%~1" == "sinc1024" (
   SET tmpinterpolate=i7
   SET tmpinterp=sinc1024
   goto :eof
 )


REM Return from subroutine
goto :eof

REM ****************************************************************************
REM Verify that we are running cmd.exe and NOT command.com.
REM We need the extra commands found in cmd.exe.
REM Also verify that Win 2000/XP are running, which have the
REM cmd.exe command interpreter.
REM Note, this subroutine cannot be 'called', you have to use 'goto'
REM because we do not know yet whether the command interpreter
REM can handle it.
REM Also, we cannot use equ, neq, gtr etc.. with 'if' statement because
REM we do not know yet if command extensions are active.
:verifyos

REM This program uses some of cmd.exe's new commands such
REM as 'set /p' and others.
REM Also verify that cmd.exe command extensions are ON

REM Note that Command.com does not have a findstr command,
REM so we must use find instead here until we know that
REM cmd.exe is running.
echo %comspec% | find /i "cmd.exe" > nul

REM Note that if we are running COMMAND.COM, we cannot use
REM '%errorlevel% gtr' command here!
if errorlevel == 2 goto wrongcom
if errorlevel == 1 goto wrongcom
REM if errorlevel == 0 then cmd.exe is running


REM Test to verify that DOS command extensions are active
REM so that we can use them in this program.
REM The 'set /a' is a command extension, if it works, then extensions
REM are active. Do not use 'if xxx neq' here.
set t1=10
set t2=
set /a t2=%t1%/2
if not a%t2%a == a5a goto extoff


REM Check OS version. Must be 2000 or XP. Why?
REM Because the SET /P command (which we use later to get user input) is only on 2000 / XP
REM 

REM Check for Windows XP
REM /C means to use search string literally - which means that 
REM the space in " XP" is included in the search. 
REM echo %vers% | findstr /i /C:" XP" > nul
ver | findstr /i /C:" XP" > nul
if %errorlevel% equ 0 goto osok

REM Check for Windows 2000
REM echo %vers% | findstr /i /C:" 2000" > nul
ver | findstr /i /C:" 2000" > nul
if %errorlevel% equ 0 goto osok

goto wrongos

:osok

REM WINDOWS XP/2000 and cmd.exe detected, so continue.
goto returnverifyos

REM ****************************************************************
:wrongos
echo Error! This DOS program will only run under Win XP or Win 2000.
echo.
pause
REM We cannot use goto :eof because of wrong OS
goto done

REM ******************************************************************
:wrongcom
echo cmd.exe is currently not running, but is required
echo for this program. 
echo The cmd.exe DOS shell runs under Windows 2000/XP.
echo Program aborted.
echo.
pause
REM We cannot use goto :eof because of wrong OS
goto done

REM *******************************************************************
:extoff
echo cmd.exe DOS interpreter command extensions are off.
echo Make sure you are running Windows 2000/XP and
echo that your DOS command interpreter is cmd.exe with extensions ON.
echo Program aborted.
echo.
pause
goto done

REM *******************************************************************************************************
REM We do not use the 'exit' command so as not to close the DOS window when the
REM user manually opened a DOS window in the first place.
:done


